import logging
from pyramid.i18n import TranslationStringFactory
from ringo.lib.i18n import translators
from ringo.lib.extension import register_modul
from ringo.lib.helpers import dynamic_import

# This import is needed to trigger "registering" the views.
import ringo_file.views

# Import models so that alembic is able to autogenerate migrations
# scripts.
from ringo_file.model import File

log = logging.getLogger(__name__)

modul_config = {
    "name": "file",
    "label": "",
    "clazzpath": "ringo_file.model.File",
    "label_plural": "",
    "str_repr": "",
    "display": "",
    "actions": ["list", "read", "update", "create", "delete"]
}


def includeme(config):
    """Registers a new modul for ringo.

    :config: Dictionary with configuration of the new modul

    """
    modul = register_modul(config, modul_config)
    if modul:
        File._modul_id = modul.get_value("id")
        translators.append(TranslationStringFactory('ringo_file'))
        config.add_translation_dirs('ringo_file:locale/')



from sqlalchemy.ext.declarative import declared_attr
import sqlalchemy as sa
from ringo.model import Base
from ringo.model.base import BaseItem, BaseFactory
from ringo.model.mixins import Owned


class FileFactory(BaseFactory):

    def create(self, user=None, values=None):
        new_item = BaseFactory.create(self, user, values)
        return new_item


class File(BaseItem, Owned, Base):
    """Docstring for file extension"""

    __tablename__ = 'files'
    """Name of the table in the database for this modul. Do not
    change!"""
    _modul_id = None
    """Will be set dynamically. See include me of this modul"""

    # Define columns of the table in the database
    id = sa.Column(sa.Integer, primary_key=True)
    name = sa.Column('name', sa.Text, nullable=True, default=None)
    data = sa.Column('data', sa.LargeBinary, nullable=True, default=None)
    description = sa.Column('description', sa.Text,
                            nullable=True, default=None)
    size = sa.Column('size', sa.Integer, nullable=True, default=None)
    mime = sa.Column('mime', sa.Text, nullable=True, default=None)

    @classmethod
    def get_item_factory(cls):
        return FileFactory(cls)


class Filed(object):
    """Mixin to make items of other modules fileable. This means the
    will get a relation named files linked to files attached to the item."""

    @declared_attr
    def files(cls):
        clsname = cls.__name__.lower()
        tbl_name = "nm_%s_files" % clsname
        nm_table = sa.Table(tbl_name, Base.metadata,
                            sa.Column('%s_id' % clsname, sa.Integer,
                                      sa.ForeignKey(cls.id)),
                            sa.Column('file_id', sa.Integer,
                                      sa.ForeignKey("files.id")))
        files = sa.orm.relationship(File, secondary=nm_table,
                                    single_parent=True,
                                    backref="items")
        return files


# Delete orphaned files. See details on:
# http://stackoverflow.com/questions/9234082/setting-delete-orphan-on-sqlalchemy-relationship-causes-assertionerror-this-att
# after_flush did not work so clean up right before the commit.
@sa.event.listens_for(sa.orm.Session, 'before_commit')
def delete_tag_orphans(session):
        session.query(File).\
        filter(~File.items.any()).\
        delete(synchronize_session=False)



# Define your custom views here or overwrite the default views. Default
# CRUD operations are generated by the ringo frameworkd.

import logging
import mimetypes

from ringo.views.base import create, update, read
from ringo.views.base import web_action_view_mapping

log = logging.getLogger(__name__)


def save_file(request, item):
    """Helper function which is called after the validation of the form
    succeeds. The function will get the data from the file from the
    request and set it in the model including size and mime type.
    Addiotionally it will set the filename based on the uploaded file if
    no other name is given."""
    try:
        #  FIXME: Use global helper method for this??? (ti) <2015-01-31
        #  16:49>
        # Rewind file
        request.POST.get('file').file.seek(0)
        data = request.POST.get('file').file.read()
        filename = request.POST.get('file').filename
        item.data = data
        item.size = len(data)
        item.mime = mimetypes.guess_type(filename)[0]
        if not request.POST.get('name'):
            item.name = filename
    except AttributeError:
        # Will be raised if the user submits no file.
        pass
    return item


def create_(request):
    return create(request, callback=save_file)


def update_(request):
    return update(request, callback=save_file)


def download(request):
    result = read(request)
    item = result['item']
    response = request.response
    response.content_type = str(item.mime)
    extension = mimetypes.guess_extension(item.mime)
    label = item.get_value("name", expand=True)
    filename = "%(filename)s%(suffix)s" % {"filename": label,
                                           "suffix": extension}
    response.content_disposition = 'attachment; filename="%s"' % filename
    response.body = item.data
    return response


web_action_view_mapping["default"]["download"] = download
# FIXME: 2016-04-19: Tried to overwrite the routing with view_config but
# without success. Looked in ringo_printtemplates how it is done there.
web_action_view_mapping["files"] = {}
web_action_view_mapping["files"]["create"] = create_
web_action_view_mapping["files"]["update"] = update_
web_action_view_mapping["files"]["download"] = download
